/*
* IntersectionTrigger v1.1.4 
* IntersectionTrigger utilizes the most modern web technology to trigger anything by intersection. Including scroll-based animations.
* https://sunshine-themes.com/?appID=ss_app_1
*
* Copyright 2023, Sunshine. All rights reserved.
* @license: Released under the personal 'no charge' license, Licensees of commercial or business license are granted additional rights. See http://sunshine-themes.com/?appID=ss_app_1&tab=license for details..
* @author: Sherif Magdy, eng.sherifmagdy.w@gmail.com
*
* Released on: May 7, 2023
*/

"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/core/core.ts
var core_exports = {};
__export(core_exports, {
  default: () => core_default
});
module.exports = __toCommonJS(core_exports);

// src/constants.ts
var fn = () => {
};
var defaultInsOptions = {
  //Defaults for every trigger
  defaults: {
    enter: "0%",
    leave: "100%",
    once: false,
    onEnter: fn,
    onLeave: fn,
    onEnterBack: fn,
    onLeaveBack: fn,
    toggleClass: void 0,
    animation: void 0
  },
  rootEnter: "100%",
  rootLeave: "0%",
  axis: "y",
  name: "",
  root: null,
  guides: false,
  onScroll: fn
};
var triggerStates = {
  hasEntered: false,
  hasEnteredBack: false,
  hasLeft: true,
  hasLeftBack: true,
  hasEnteredOnce: false,
  onScroll: { backup: void 0, animate: void 0 },
  ids: { snapTimeOutId: 0 }
};

// src/helpers.ts
var is = {
  function: (a) => typeof a === "function",
  string: (a) => "string" === typeof a,
  boolean: (a) => "boolean" === typeof a,
  object: (a) => !!a && "object" === typeof a && a !== null && !(a instanceof Array),
  inObject: (obj, prop) => Object.prototype.hasOwnProperty.call(obj, prop),
  num: (a) => typeof a === "number",
  array: (a) => a instanceof Array,
  element: (a) => a instanceof HTMLElement,
  empty: (a) => Object.keys(a).length === 0,
  doc: (a) => a instanceof Document,
  anime: (a) => is.object(a) && is.inObject(a, "animatables") && !is.inObject(a, "add"),
  tl: (a) => is.object(a) && is.inObject(a, "add") && is.function(a.add),
  animeInstance: (a) => is.anime(a) || is.tl(a),
  pixel: (a) => a.includes("px"),
  percent: (a) => a.includes("%"),
  scrollable: (element, dir) => dir ? "y" === dir ? element.scrollHeight > element.clientHeight : element.scrollWidth > element.clientWidth : element.scrollHeight > element.clientHeight || element.scrollWidth > element.clientWidth
};
var roundFloat = (value, precision) => {
  is.string(value) && (value = parseFloat(value));
  const multiplier = Math.pow(10, precision || 0);
  return Math.round(value * multiplier) / multiplier;
};
var deepClone = (obj) => {
  if (!is.object(obj) && !is.array(obj) || is.animeInstance(obj) || obj instanceof Element)
    return obj;
  const clone = is.array(obj) ? [] : {};
  for (const k in obj) {
    if (is.inObject(obj, k))
      clone[k] = deepClone(obj[k]);
  }
  return clone;
};
var mergeOptions = (defaultOptions, customOptions) => {
  const options = { ...defaultOptions };
  for (const [key, value] of Object.entries(customOptions)) {
    const k = key;
    if (is.object(options[k]) && !is.empty(options[k])) {
      if (!is.object(value))
        continue;
      options[k] = mergeOptions(options[k], value);
    } else {
      options[k] = value;
    }
  }
  return options;
};
var throwError = (message) => {
  throw new Error(message);
};
var getMinMax = (n1, n2) => [n1, n2].sort((a, b) => a - b);
var parseValue = (v) => {
  let output = { value: 0, unit: "" };
  const parts = /^(-?\d*\.?\d+)(px|%)$/.exec(v);
  parts && (output = { value: parseFloat(parts[1]), unit: parts[2] });
  return output;
};
var parseString = (str) => str.split(/\s+/).map((v) => parseValue(v));

// src/utils/utils.ts
var Utils = class {
  _it;
  tD;
  states;
  isVertical;
  getRoot;
  dirProps;
  setRootMargin;
  setThreshold;
  parseQuery;
  parseRoot;
  validatePosition;
  setPositionData;
  parsePositions;
  deleteTriggerData;
  hasTriggerData;
  getTriggerData;
  setTriggerData;
  setTriggerStates;
  setTriggerScrollStates;
  triggerEvent;
  getPositions;
  toggleActions;
  parseRootMargin;
  getRootRect;
  expandRectByRootMargin;
  constructor(intersectionTrigger) {
    this._it = intersectionTrigger;
    this.tD = this._it._triggersData;
    this.states = this._it._states;
    this.setUtils();
    return this;
  }
  setUtils() {
    this.isVertical = () => "y" === this._it.axis;
    this.getRoot = (forEvent) => {
      if (!this._it._root) {
        if (forEvent === "resize")
          return window;
        if (forEvent === "scroll")
          return document;
        return document.documentElement;
      }
      return this._it._root;
    };
    this.dirProps = () => this.isVertical() ? { ref: "top", length: "height", refOpposite: "bottom", clientLength: document.documentElement.clientHeight } : { ref: "left", length: "width", refOpposite: "right", clientLength: document.documentElement.clientWidth };
    this.setRootMargin = (rEP, rLP) => {
      const { length, clientLength } = this.dirProps();
      const rootLength = this._it._root ? this._it._root.getBoundingClientRect()[length] : clientLength;
      const valueToPx = (pos, total) => {
        const { value, unit, normal } = pos;
        if ("%" === unit)
          return normal * total;
        return value;
      };
      rEP.pixels = valueToPx(rEP, rootLength);
      rLP.pixels = valueToPx(rLP, rootLength);
      this._it._isREPGreater = rEP.pixels >= rLP.pixels;
      const rootMargins = {
        fromRef: `${-1 * (this._it._isREPGreater ? rLP.pixels : rEP.pixels)}px`,
        //root margin from direction Reference (top|left)
        fromOppRef: `${(this._it._isREPGreater ? rEP.pixels : rLP.pixels) - rootLength}px`
        //root margin from the direction Reference's opposite (bottom|right)
      };
      const root = this.getRoot();
      const extendMargin = this.isVertical() ? root.scrollWidth : root.scrollHeight;
      return this.isVertical() ? `${rootMargins.fromRef} ${extendMargin}px ${rootMargins.fromOppRef} ${extendMargin}px` : `${extendMargin}px ${rootMargins.fromOppRef} ${extendMargin}px ${rootMargins.fromRef}`;
    };
    this.setThreshold = () => {
      const threshold = [0, 1];
      this._it.triggers.forEach((trigger) => {
        const { enter, leave, maxPosition } = this.getTriggerData(trigger);
        threshold.push(enter, leave, roundFloat(1 - maxPosition, 2));
      });
      return [...new Set(threshold)];
    };
    this.parseQuery = (q, errLog = "trigger") => {
      if (is.string(q))
        return [...document.querySelectorAll(q)];
      if (is.array(q))
        return q;
      if (is.element(q))
        return [q];
      return throwError(`${errLog} parameter must be a valid selector, an element or array of elements`);
    };
    this.parseRoot = (query) => {
      if (!query)
        return null;
      if (is.string(query)) {
        const el = document.querySelector(query);
        if (!el)
          return throwError("root parameter must be a valid selector");
        return el;
      }
      if (is.element(query))
        return query;
      return throwError("root parameter must be an element");
    };
    this.validatePosition = (pos) => {
      is.function(pos) && (pos = pos(this._it));
      if (!is.string(pos))
        return throwError(`enter, leave, rootEnter and rootLeave parameters must be a string.`);
      return pos;
    };
    this.setPositionData = (pos) => {
      pos = this.validatePosition(pos);
      const original = pos.trim();
      const parsed = parseValue(original);
      const roundedValue = roundFloat(parsed.value);
      return {
        original,
        unit: parsed.unit,
        value: roundedValue,
        normal: parsed.unit === "%" ? roundedValue / 100 : 0
      };
    };
    this.parsePositions = (triggerEnter, triggerLeave, rootEnter, rootLeave) => {
      const positionsData = [triggerEnter, rootEnter, triggerLeave, rootLeave].map(
        (pos) => this.setPositionData(this.validatePosition(pos).trim())
      );
      return {
        tEP: positionsData[0],
        //trigger enter position
        rEP: positionsData[1],
        //root enter position
        tLP: positionsData[2],
        //trigger leave position
        rLP: positionsData[3]
        //root leave position
      };
    };
    this.deleteTriggerData = (trigger) => this.tD.delete(trigger);
    this.getTriggerData = (trigger, prop) => {
      const triggerData = this.tD.get(trigger) || {};
      if (prop)
        return triggerData[prop];
      return triggerData;
    };
    this.setTriggerData = (trigger, value, isPartial) => {
      const storedValue = this.getTriggerData(trigger);
      const newValue = isPartial ? { ...storedValue, ...value } : value;
      this.tD.set(trigger, newValue);
    };
    this.setTriggerStates = (trigger, value) => {
      const triggerStates2 = this.getTriggerData(trigger, "states");
      this.setTriggerData(trigger, { states: { ...triggerStates2, ...value } }, true);
    };
    this.setTriggerScrollStates = (trigger, prop, value) => {
      const triggerScrollStates = this.getTriggerData(trigger, "states").onScroll;
      triggerScrollStates[prop] = value;
      this.setTriggerStates(trigger, { onScroll: triggerScrollStates });
      if (value) {
        if (0 === this.states.runningScrollCbs)
          this._it.addScrollListener(this._it._onScrollHandler);
        this.states.runningScrollCbs++;
        return;
      }
      if (0 < this.states.runningScrollCbs)
        this.states.runningScrollCbs--;
      if (0 === this.states.runningScrollCbs)
        this._it.removeScrollListener(this._it._onScrollHandler);
    };
    this.triggerEvent = (trigger, [name, callback, enterState, leaveState, index]) => {
      const {
        once,
        toggleClass,
        animation,
        states: { hasEnteredOnce }
      } = this.getTriggerData(trigger);
      callback(trigger, this._it);
      if (this._it.killed)
        return this.kill();
      toggleClass && this._it.toggleClass.toggle(trigger, toggleClass, index);
      animation && this._it.animation.animate(trigger, animation, index);
      const isEnterEvent = "Enter" === name || "EnterBack" === name;
      let triggerStates2 = {};
      if (isEnterEvent) {
        triggerStates2 = {
          [enterState]: true,
          [leaveState]: false
        };
        if (!hasEnteredOnce)
          triggerStates2 = { [enterState]: true, hasLeft: false, hasLeftBack: false, hasEnteredOnce: true };
      } else {
        if (once && hasEnteredOnce) {
          this._it.remove(trigger);
          return;
        }
        triggerStates2 = {
          [leaveState]: true,
          hasEntered: false,
          hasEnteredBack: false
        };
      }
      this.setTriggerStates(trigger, triggerStates2);
    };
    this.getPositions = (tB, rB, { enter, leave, ref, refOpposite, length }) => {
      const isREPGreater = this._it._isREPGreater;
      return [
        tB[ref] + enter * tB[length],
        //tEP
        tB[ref] + leave * tB[length],
        //tLP
        isREPGreater ? rB[refOpposite] : rB[ref],
        //rEP
        isREPGreater ? rB[ref] : rB[refOpposite]
        //rLP
      ];
    };
    this.toggleActions = (trigger) => {
      const { enter, leave, onEnter, onLeave, onEnterBack, onLeaveBack, states } = this.getTriggerData(trigger), tB = trigger.getBoundingClientRect(), rB = this._it.rootBounds = this.getRootRect(this._it.observer.rootMargin), { ref, refOpposite, length } = this.dirProps(), [tEP, tLP, rEP, rLP] = this.getPositions(tB, rB, { enter, leave, ref, refOpposite, length });
      let modStates = { ...states, hasEnteredFromOneSide: states.hasEntered || states.hasEnteredBack };
      const updateStates = () => {
        const states2 = this.getTriggerData(trigger, "states");
        modStates = { ...states2, hasEnteredFromOneSide: states2.hasEntered || states2.hasEnteredBack };
      };
      if (modStates.hasLeftBack && rEP > tEP) {
        this.triggerEvent(trigger, ["Enter", onEnter, "hasEntered", "hasLeftBack", 0]);
        updateStates();
      }
      if (modStates.hasLeft && modStates.hasEnteredOnce && rLP < tLP) {
        this.triggerEvent(trigger, ["EnterBack", onEnterBack, "hasEnteredBack", "hasLeft", 2]);
        updateStates();
      }
      if (modStates.hasEnteredFromOneSide && rLP > tLP)
        this.triggerEvent(trigger, ["Leave", onLeave, null, "hasLeft", 1]);
      if (modStates.hasEnteredFromOneSide && rEP < tEP)
        this.triggerEvent(trigger, ["LeaveBack", onLeaveBack, null, "hasLeftBack", 3]);
    };
    this.parseRootMargin = (rootMargins) => {
      const marginString = rootMargins || "0px";
      const margins = parseString(marginString);
      margins[1] = margins[1] || margins[0];
      margins[2] = margins[2] || margins[0];
      margins[3] = margins[3] || margins[1];
      return margins;
    };
    this.expandRectByRootMargin = (rect, rootMargins) => {
      const margins = this.parseRootMargin(rootMargins).map((margin, i) => {
        return margin.unit === "px" ? margin.value : margin.value * (i % 2 ? rect.width : rect.height) / 100;
      });
      const newRect = {
        top: rect.top - margins[0],
        right: rect.right + margins[1],
        bottom: rect.bottom + margins[2],
        left: rect.left - margins[3],
        width: 0,
        height: 0
      };
      newRect.width = newRect.right - newRect.left;
      newRect.height = newRect.bottom - newRect.top;
      return newRect;
    };
    this.getRootRect = (rootMargins) => {
      let rootRect;
      if (this._it._root && !is.doc(this._it._root)) {
        rootRect = this._it._root.getBoundingClientRect();
        return this.expandRectByRootMargin(rootRect, rootMargins);
      }
      const doc = is.doc(this._it._root) ? this._it._root : document;
      const html = doc.documentElement;
      const body = doc.body;
      rootRect = {
        top: 0,
        left: 0,
        right: html.clientWidth || body.clientWidth,
        width: html.clientWidth || body.clientWidth,
        bottom: html.clientHeight || body.clientHeight,
        height: html.clientHeight || body.clientHeight
      };
      return this.expandRectByRootMargin(rootRect, rootMargins);
    };
  }
  kill() {
    this._it = void 0;
  }
};

// src/core/core.ts
var registeredPlugins = [];
var instances = [];
var instanceID = 0;
var _IntersectionTrigger = class {
  id;
  observer;
  toggleClass;
  customScrollHandler;
  animation;
  guides;
  axis;
  name;
  triggers;
  rootBounds;
  killed;
  _states;
  _options;
  _triggersData;
  _defaultOptions;
  _userOptions;
  _onResizeHandler;
  _positionsData;
  _utils;
  _isREPGreater;
  _threshold;
  _rootMargin;
  _rAFID;
  _root;
  constructor(options) {
    this._userOptions = options || {};
    this.triggers = [];
    this._triggersData = /* @__PURE__ */ new WeakMap();
    this.id = instanceID;
    instanceID++;
    instances.push(this);
    this.animation = void 0;
    this.toggleClass = void 0;
    this.guides = void 0;
    this._states = {
      oCbFirstInvoke: true,
      //observer callback first call
      runningScrollCbs: 0
      //Running scroll functions
    };
    this._utils = new Utils(this);
    this._setInstance();
  }
  _setPlugin(pluginName) {
    const plugins = _IntersectionTrigger.getRegisteredPlugins();
    const Plugin = plugins.find((plg) => pluginName === plg.pluginName);
    Plugin && (this[pluginName] = new Plugin(this));
  }
  _addResizeListener() {
    this._removeResizeListener();
    this._onResizeHandler = () => this.update();
    this._utils.getRoot("resize").addEventListener("resize", this._onResizeHandler, false);
  }
  _removeResizeListener() {
    this._utils.getRoot("resize").removeEventListener("resize", this._onResizeHandler, false);
  }
  _rAFCallback = () => {
    this.triggers.forEach((trigger) => {
      const onScrollFuns = this._utils.getTriggerData(trigger, "states").onScroll;
      for (const k in onScrollFuns) {
        const fnName = k;
        onScrollFuns[fnName] && onScrollFuns[fnName](trigger);
      }
    });
  };
  _onScrollHandler = () => this._rAFID = requestAnimationFrame(this._rAFCallback);
  addScrollListener(handler) {
    this._utils.getRoot("scroll").addEventListener("scroll", handler, false);
  }
  removeScrollListener(handler) {
    this._utils.getRoot("scroll").removeEventListener("scroll", handler, false);
  }
  _observerCallback = (entries, observer) => {
    const { length } = this._utils.dirProps();
    for (const entry of entries) {
      const trigger = entry.target, tB = entry.boundingClientRect, isIntersecting = entry.isIntersecting;
      const rB = this.rootBounds = entry.rootBounds || this._utils.getRootRect(observer.rootMargin), rL = rB[length];
      const {
        onScroll: { backup }
      } = this._utils.getTriggerData(trigger, "states"), initBackupFun = tB[length] >= rL, isBackupFunRunning = !!backup;
      const toggleActions = this._utils.toggleActions;
      const setStates = this._utils.setTriggerScrollStates;
      toggleActions(trigger);
      if (this._states.oCbFirstInvoke) {
        isIntersecting && initBackupFun && setStates(trigger, "backup", toggleActions);
        continue;
      }
      if (isIntersecting) {
        !isBackupFunRunning && initBackupFun && setStates(trigger, "backup", toggleActions);
      } else {
        isBackupFunRunning && setStates(trigger, "backup");
      }
    }
    this._states.oCbFirstInvoke = false;
  };
  _createInstance() {
    this._rootMargin = this._utils.setRootMargin(this._positionsData.rEP, this._positionsData.rLP);
    this._threshold = this._utils.setThreshold();
    this.observer = new IntersectionObserver(this._observerCallback, {
      root: this._root,
      rootMargin: this._rootMargin,
      threshold: this._threshold
    });
    this._root = this.observer.root;
    this.rootBounds = this._utils.getRootRect(this.observer.rootMargin);
  }
  _setInstance() {
    this._defaultOptions = defaultInsOptions;
    this._options = mergeOptions(this._defaultOptions, this._userOptions);
    const {
      axis,
      name,
      root,
      defaults: { enter, leave },
      onScroll,
      rootEnter,
      rootLeave,
      guides
    } = this._options;
    this.axis = axis;
    this.name = name;
    this._root = this._utils.parseRoot(root);
    this._positionsData = this._utils.parsePositions(enter, leave, rootEnter, rootLeave);
    this.customScrollHandler = onScroll;
    this._createInstance();
    this._addResizeListener();
    this.customScrollHandler && this.addScrollListener(this.customScrollHandler);
    if (guides) {
      this._setPlugin("guides");
      this.guides.init(guides);
    }
    return this;
  }
  add(trigger, options) {
    const toAddTriggers = this._utils.parseQuery(trigger), { defaults } = this._options, userOpts = options || {};
    const getPositionNormal = (pos, name = "tEP") => pos ? this._utils.setPositionData(pos).normal : this._positionsData[name].normal, getPlugin = (name) => {
      !this[name] && this._setPlugin(name);
      return this[name];
    };
    const mergedParams = mergeOptions(defaults, userOpts), { enter, leave, toggleClass, animation } = mergedParams, triggerParams = {
      ...mergedParams,
      enter: getPositionNormal(enter),
      leave: getPositionNormal(leave, "tLP"),
      toggleClass: toggleClass ? getPlugin("toggleClass").parse(toggleClass) : void 0,
      animation: animation ? getPlugin("animation").parse(animation) : void 0,
      states: triggerStates
    };
    const [minPosition, maxPosition] = getMinMax(triggerParams.enter, triggerParams.leave);
    triggerParams.minPosition = minPosition;
    triggerParams.maxPosition = maxPosition;
    this.triggers = [.../* @__PURE__ */ new Set([...this.triggers, ...toAddTriggers])];
    let mustUpdate = false;
    [triggerParams.enter, triggerParams.leave].forEach(
      (normalizedPos) => !this._threshold.some((value) => normalizedPos === value) && (mustUpdate = true)
    );
    if (mustUpdate) {
      toAddTriggers.forEach((trigger2) => this._utils.setTriggerData(trigger2, deepClone(triggerParams)));
      this.update();
    } else {
      toAddTriggers.forEach((trigger2) => {
        this._utils.setTriggerData(trigger2, deepClone(triggerParams));
        this.observer.observe(trigger2);
      });
    }
    this.guides && this.guides.update();
    return this;
  }
  remove(trigger) {
    const toRemoveTriggers = this._utils.parseQuery(trigger);
    toRemoveTriggers.forEach((trigger2) => {
      this._utils.deleteTriggerData(trigger2);
      this.observer.unobserve(trigger2);
    });
    const updatedStoredTriggers = this.triggers.filter((storedTrigger) => {
      const isInRemoveTriggers = toRemoveTriggers.some((toRemoveTrigger) => storedTrigger === toRemoveTrigger);
      return !isInRemoveTriggers;
    });
    this.triggers = updatedStoredTriggers;
    this.guides && this.guides.update();
    return this;
  }
  _disconnect() {
    this.observer && this.observer.disconnect();
    this.observer = void 0;
  }
  update() {
    this._disconnect();
    this._createInstance();
    this.triggers.forEach((trigger) => this.observer && this.observer.observe(trigger));
    this.guides && this.guides.update();
  }
  kill() {
    this.killed = true;
    this._disconnect();
    this.removeScrollListener(this._onScrollHandler);
    this.removeScrollListener(this.customScrollHandler);
    this._removeResizeListener();
    this._rAFID && cancelAnimationFrame(this._rAFID);
    this.guides && this.guides.kill();
    this.toggleClass && this.toggleClass.kill();
    this.animation && this.animation.kill();
    this._utils.kill();
    this.triggers = [];
    this.animation = this.toggleClass = this.guides = this._utils = void 0;
    const instanceIndex = instances.indexOf(this);
    ~instanceIndex && instances.splice(instanceIndex, 1);
  }
};
var IntersectionTrigger = _IntersectionTrigger;
__publicField(IntersectionTrigger, "getInstanceById");
__publicField(IntersectionTrigger, "registerPlugins");
__publicField(IntersectionTrigger, "getInstances");
__publicField(IntersectionTrigger, "getRegisteredPlugins");
__publicField(IntersectionTrigger, "update");
__publicField(IntersectionTrigger, "kill");
IntersectionTrigger.getInstances = () => instances;
IntersectionTrigger.getInstanceById = (id) => instances.find((ins) => ins.id === id);
IntersectionTrigger.update = () => instances.forEach((ins) => ins.update());
IntersectionTrigger.kill = () => {
  while (instances.length)
    instances[0].kill();
};
IntersectionTrigger.registerPlugins = (plugins = []) => registeredPlugins.push(...plugins);
IntersectionTrigger.getRegisteredPlugins = () => registeredPlugins;
var core_default = IntersectionTrigger;
